Documentation
¶
Overview ¶
Package chainedhotstuff implements the pipelined three-chain version of the HotStuff protocol.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
Types ¶
type ChainedHotStuff ¶ added in v0.3.0
type ChainedHotStuff struct {
// contains filtered or unexported fields
}
ChainedHotStuff implements the pipelined three-phase HotStuff protocol.
func (*ChainedHotStuff) ChainLength ¶ added in v0.5.0
func (hs *ChainedHotStuff) ChainLength() int
ChainLength returns the number of blocks that need to be chained together in order to commit.
func (*ChainedHotStuff) CommitRule ¶ added in v0.3.0
func (hs *ChainedHotStuff) CommitRule(block *hotstuff.Block) *hotstuff.Block
CommitRule decides whether an ancestor of the block should be committed.
func (*ChainedHotStuff) InitModule ¶ added in v0.5.0
func (hs *ChainedHotStuff) InitModule(mods *modules.Core)
InitModule initializes the module.
func (*ChainedHotStuff) VoteRule ¶ added in v0.3.0
func (hs *ChainedHotStuff) VoteRule(proposal hotstuff.ProposeMsg) bool
VoteRule decides whether to vote for the proposal or not.
Click to show internal directories.
Click to hide internal directories.